@@ -125,17 +125,20 @@ int TranslateSQLType(int stp, int prec, int& len, char& v, bool& w)
   switch (stp) {
     case SQL_WVARCHAR:                      //  (-9)
       w = true;
+      /* fall through */
     case SQL_VARCHAR:                       //   12
       v = 'V';
       type = TYPE_STRING;
       break;
     case SQL_WCHAR:                         //  (-8)
       w = true;
+      /* fall through */
     case SQL_CHAR:                          //    1
       type = TYPE_STRING;
       break;
     case SQL_WLONGVARCHAR:                  // (-10)
       w = true;
+      /* fall through */
     case SQL_LONGVARCHAR:                   //  (-1)
 			if (GetTypeConv() == TPC_YES || GetTypeConv() == TPC_FORCE) {
 				v = 'V';
@@ -997,6 +1000,11 @@ ODBConn::ODBConn(PGLOBAL g, TDBODBC *tdbp)
   m_Full = false;
   m_UseCnc = false;
   m_IDQuoteChar[0] = '"';
+  if (tdbp)
+  {
+    if (tdbp->Quoted && tdbp->Quote)
+      m_IDQuoteChar[0] = *tdbp->Quote;
+  }
   m_IDQuoteChar[1] = 0;
 //*m_ErrMsg = '\0';
   } // end of ODBConn
@@ -1179,6 +1187,7 @@ int ODBConn::Open(PCSZ ConnectString, POPARM sop, DWORD options)
     // Verify support for required functionality and cache info
 //  VerifyConnect();         Deprecated
     GetConnectInfo();
+    // Still we want to use the set QChar
   } catch(DBX *xp) {
     snprintf(g->Message, sizeof(g->Message), "%s: %s", xp->m_Msg, xp->GetErrorMessage(0));
     Close();
@@ -2349,6 +2358,7 @@ int ODBConn::GetCatInfo(CATPARM *cap)
         break;
       case CAT_SPC:
         ThrowDBX("SQLSpecialColumns not available yet");
+        break;
       default:
         ThrowDBX("Invalid SQL function id");
       } // endswitch infotype
